// conversion.cpp: calculadora
// Fecha: 8/10/2020; 8:07 pm
// S. Galliano

#include "conversion.h"
#include "menu.h"
#include <cmath> // para fmod()
using std::make_tuple;

//  dispone todos los valores
void Conver::factorizar( short opcion, double valor ) {
    double k; // factor primario de conversin

    switch ( opcion ) {
    case VolLiq:
        k = 3.785412;
        param = make_tuple( valor, valor / k, valor * k, "gal", "L  " );
        break;
    case VolSol:
        k = 0.028317;
        param = make_tuple( valor, valor / k, valor * k, "ft3", "m3 " );
        break;
    case Masa:
        k = 0.453593;
        param = make_tuple( valor, valor / k, valor * k, "lb", "kg" );
        break;
    case Area:
        k = 0.404686;
        param = make_tuple( valor, valor / k, valor * k, "ac", "ha" );
        break;
    case Largo:
        k = 1.609344;
        param = make_tuple( valor, valor / k, valor * k, "mi", "km" );
        break;
    case Temp:
        k = 32.0;
        param = make_tuple( valor, 1.8 * valor + k, ( valor - k ) / 1.8, "F", "C" );
        break;
    case Ang:
        k = 45.0 / atan( 1.0 );
        // eliminar las vueltas completas
        double vmt = valor * k;
        if ( vmt > 360.0 ) vmt = fmod( vmt, 360.0 );
        param = make_tuple( valor, valor / k, vmt, "rad  ", "grado" );
        break;
    } // switch
}

#include <iostream>
#include <iomanip>

void Conver::mostrar() const {
    int ancho   = 8;
    int decimal = 4;
    double val,   // valor a convertir
           valEN, // valor convertido a ingls
           valMT; // valor convertido a mtrico
    Str    uniEN, // unidad inglesa
           uniMT; // unidad mtrica

    tie( val, valEN, valMT, uniEN, uniMT ) = param;

    std::ios::fmtflags oldFlags = std::cout.flags(); // formato original
    std::cout
    << setiosflags( std::ios::fixed | std::ios::showpoint ) << std::setprecision( decimal );

    //  primera lnea a consola
    std::cout << '\n' <<
    std::setw( ancho )  << val   << ' ' << uniEN << " --> " <<
    std::setw( ancho )  << valMT << ' ' << uniMT;

    //  segunda lnea a consola
    std::cout << '\n' <<
    std::setw( ancho )  << val   << ' ' << uniMT << " --> " <<
    std::setw( ancho )  << valEN << ' ' << uniEN;

    std::cout.unsetf( std::ios::fixed | std::ios::showpoint );
    std::cout.flags( oldFlags ); // formato original restaurado
}
