//  almacen.cpp: una bd
//  Fecha: 11/10/2020; 8:13 am
//  S. Galliano
#include "almacen.h"
#include <iostream>

Str ERR_S( "Error. Esa marca ya est\240.\n\n" );
Str ERR_N( "Error. Esa marca no est\240.\n\n" );
Str HECHO( "Hecho.\n\n" );

Almacen::Almacen() {
    Data d1( "Mocha Munch",   65, 350, 208,  .9  );
    Data d2( "Krispy Krunch", 70, 340, 411, 1.05 );
    Data d3( "Honey Beez",    90, 390, 789, 1.25 );
    stock.push_back( d1 );
    stock.push_back( d2 );
    stock.push_back( d3 );
}

//  comprueba si una marca est en la BD
bool Almacen::marcaEnStock( Str mk ) {
    for ( auto x: stock )           // se visita el vector
        if ( mk == std::get<0>(x) ) // si est ...
            return true;            // ... regresa true

    return false; // si no, regresa false
}

using std::cerr;
using std::cout;

void Almacen::vaciar() {
    stock.empty();
    cout << HECHO;
}

void Almacen::adicionar( Str mk ) {
    if ( marcaEnStock( mk ) )
        cerr << ERR_S;
    else {
        Dulce d;
        Data tmp = d.entrarDato( mk );
        stock.push_back(tmp);
        cout << HECHO;
    }
}

using std::cin;

void Almacen::editar( Str mk ) {
    if ( not marcaEnStock( mk ) ) {
        cerr << ERR_N;
        return;
    }

    Stock::iterator i = stock.begin();

    for ( auto x: stock ) {
        if ( mk == std::get<0>( x ) ) {
            Str valor;
            int cl    = std::get<1>( x );
            int ps    = std::get<2>( x );
            int ct    = std::get<3>( x );
            double pr = std::get<4>( x );
            cout << "Entrar un espacio en blanco\nen lo que no va a cambiar.";
            cout << "\nPeso = " << ps
                 << "\nNuevo peso (g): ";
            cin >> valor;
            if ( not ( " " == valor ) )  ps = atoi(valor.c_str());

            cout << "\nCalor\241as = " << cl
                 << "\nNuevas calor\241as (cal): ";
            cin >> valor;
            if ( not ( " " == valor ) ) cl = atoi( valor.c_str() );

            cout << "\nPrecio = " << pr
                 << "\nNuevo precio (USD): ";
            cin >> valor;
            if ( not ( " " == valor ) ) pr = atof( valor.c_str() );

            cout << "\nUnidades = " << ct
                 << "\nNuevas unidades (u): ";
            cin >> valor;
            if ( not ( " " == valor ) ) ct = atoi( valor.c_str() );

            Data tmp = std::make_tuple( mk, cl, ps, ct, pr );
            stock.erase( i );
            stock.insert( i, tmp );
            cout << HECHO;
            return;
        } // if

        ++i;
    }
}

void Almacen::borrar( Str mk ) {
    if ( not marcaEnStock( mk ) ) {
        cerr << ERR_N;
        return;
    }

    Stock::iterator i = stock.begin();

    for ( auto x: stock ) {
        if ( mk == std::get<0>( x ) ) {
            stock.erase( i );
            cout << HECHO;
            return;
        } // if

        ++i;
    }
}

#include <iomanip>
#include <ios>
using std::setfill;
using std::setw;

void encabeza( char ch ) {
    cout << setw(50) << setfill( ch ) << '\n' << setfill( ' ' );
}

void Almacen::mostrar() {
    encabeza( '=' );
    std::ios::fmtflags oldFlags = cout.flags();
    cout << std::left  << setw( 16 ) << "Marca"
         << std::right << std::setw( 10 )  << "Peso"
         << setw( 8 ) << "Cals."
         << setw( 8 ) << "Cant."
         << setw( 8 ) << "Precio\n";
    encabeza( '-' );
    Dulce tmp;
    double totPR = 0;
    int totCT = 0;

    for ( auto x: stock ) {
        totPR += std::get<4>(x) * std::get<3>(x);
        totCT += std::get<3>(x);
        tmp.setDato(x);    // se visita el vector
        tmp.mostrarDato(); // se muestra en consola
    }

    encabeza( '=' );
    cout.setf( std::ios_base::fixed );
    cout << "\nHay " << stock.size() << " marcas diferentes,"
         << "\ncon un total de " << totCT << " u en almac\202n,"
         << "\npara un total de " << std::setprecision( 2 ) << totPR << " CUC\n\n";
    cout.unsetf( std::ios_base::fixed );
    cout.flags( oldFlags );
}
