#ifndef VARRAY_H_INCLUDED
#define VARRAY_H_INCLUDED

//  varray.h: declaraciones para las plantillas del valarray
//  Fecha: 2/5/2020; 11:50 pm
//  Por S. Galliano.

#include <iostream>
#include <iomanip>
#include <valarray>

/**
@brief VA: convierte un slice (o subconjunto) en un valarray.
Adaptado de Nicolai Josuttis, 1999. Galliano, 2019.
@param <b>vaSlice:</b> un subconjunto o <em>slice</em>.
@return un valarray
*/
template <typename T> inline
std::valarray<typename T::value_type> VA(const T &vaSlice) {
    return std::valarray<typename T::value_type>(vaSlice);
}

/**
@brief PRINTVA: imprime el slice de un valarray.
Adaptado de Nicolai Josuttis, 1999. Galliano, 2019
@param <b>vaSlice:</b> un subconjunto o <em>slice</em>.
@return imprime los valores.
*/
template<typename T> inline
void PRINTVA(const T &vaSlice) {
    std::ios::fmtflags oldFlags = std::cout.flags();
    std::cout << std::fixed;

    for ( unsigned i = 0; i < vaSlice.size(); i++ )
        std::cout << std::setw(8) << vaSlice[i] << '\n';

    std::cout.flags(oldFlags);
}

/**
@brief PRINTVA: imprime un valarray según las columnas pasadas.
Adaptado de Nicolai Josuttis, 1999. Galliano, 2019.
@param <b>vaSlice:</b> un valarray.
@param <b>cc:</b> número de columnas a consola.
@param <b>st:</b> encabezamiento de cada fila; <em>por omisión es nulo</em>.
@return imprime los valores en una tabla a cc columnas.
*/
template<class T> inline
void PRINTVA(const T &vaSlice, int cc, std::string st = "") {
    std::ios::fmtflags oldFlags = std::cout.flags();
    std::cout << std::fixed;

    for ( unsigned i = 0; i < vaSlice.size() / cc; ++i ) {
        if ( st.size() > 0 )
            std::cout << st << i + 1;

        for ( int j = 0; j < cc; ++j )
            std::cout << std::setw(8) << vaSlice[i * cc + j];

        if ( vaSlice.size() / cc > 1 )
            std::cout << '\n';
    } // for unsigned i

    std::cout.flags(oldFlags);
}

#endif // VARRAY_H_INCLUDED
