//  vectores.cpp: definiciones para el programa 4
//  Fecha: 25/7/2020; 11 am
//  Revisión: 14/11/2020; 12:50 pm
//  Por: S. Galliano.

#include "vectores.h"
#include <iomanip>
#include <ios>
#include <iostream>

using std::cout;
using std::cin;
using std::string;

namespace miVector { // <--
// el constructor parametrizado toma los valores y los fija
Vector::Vector( double a, double b ) : x( a ), y( b ), modo(Cartesiano) {}

// operaciones: son la suma y resta, el producto interno,
// el de un vector por un escalar y la negación del vector
Vector Vector::operator-() const { return Vector( -x, -y ); }                       // -A
Vector Vector::operator+( Vector & b ) const { return Vector( x + b.x, y + b.y ); } // A + B
Vector Vector::operator-( Vector & b ) const { return Vector( x - b.x, y - b.y ); } // A - B
Vector Vector::operator*( double n )   const { return Vector( n*x, n*y ); }         // n x A
void   Vector::operator=( Modo valor ) { modo = valor; }                            // A = modo

// para completamiento de la interfaz
Vector operator*( double n, const Vector & a ) { return a * n; }        // A x n
double modulo( const Vector & v ) { return sqrt( v.x*v.x + v.y*v.y ); } // |A|

// ángulo sita (rad.)
double angulo( const Vector & v ) {
    double resp;

    if ( 0 == v.x ) {
        resp = 0;
    } else {
        resp = atan2( v.y, v.x );
        resp *= GRAD2RAD;
    } // if-else

    return resp;
}

// coseno entre vectores
double coseno( Vector & a, Vector & b ) { return ( a*b ) / ( modulo( a ) * modulo( b ) ); }
double operator*( const Vector & a, const Vector & b ) { return ( a.x*b.x + a.y*b.y ); } // A.B

// muestra un vector en su formato, dependiendo del modo
std::ostream & operator<<( std::ostream & os, const Vector & v ) {
    std::ios::fmtflags oldFlags = os.flags();
    os << std::setprecision( 2 );

    if ( Cartesiano == v.getModo() ) {
        os << "(" << v.x << ", " << v.y << ")";
    } else {
        os << "(" << modulo( v )  << ", ";
        os << std::setprecision( 3 );
        os << angulo( v ) << ")";
    } // if-else

    os.flags( oldFlags );
    return os;
}

std::istream & operator>>( std::istream & is, Vector & v ){
    char parentesis, coma;
    double x, y;
    is.setf( std::ios::skipws );   // evita los espacios en blanco
    is >> parentesis;              // lee el paréntesis
    is >> x;                       // lee la 1ra coordenada
    is >> coma;                    // lee la coma
    is >> y;                       // lee la 2da coordenada;
    is.ignore( 255, '\n' );        // deja limpio el búfer
    is.unsetf( std::ios::skipws ); // retorna al valor por defecto
    v.setValX( x );
    v.setValY( y );
    v = Cartesiano;
    return is;
}
} // namespace miVector <--
